@@ -26,6 +26,35 @@ _allprops = []
 _seenprops = []
 
 
+class XMLManualAction(object):
+    """
+    Helper class for tracking and performing the user requested manual
+    XML action
+    """
+    ACTION_CREATE = 1
+    ACTION_DELETE = 2
+    ACTION_SET = 3
+    def __init__(self, xpath, value=None, action=-1):
+        self.xpath = xpath
+        self._value = value
+
+        self._action = self.ACTION_SET
+        if action != -1:
+            self._action = action
+
+    def perform(self, xmlstate):
+        xpath = self.xpath
+        if xpath.startswith("."):
+            xpath = xmlstate.make_abs_xpath(self.xpath)
+        if self._action == self.ACTION_DELETE:
+            setval = False
+        elif self._action == self.ACTION_CREATE:
+            setval = True
+        else:
+            setval = self._value
+        xmlstate.xmlapi.set_xpath_content(xpath, setval)
+
+
 class _XMLPropertyCache(object):
     """
     Cache lookup tables mapping classes to their associated
@@ -192,8 +221,8 @@ class XMLProperty(_XMLPropertyBase):
         :param do_abspath: If True, run os.path.abspath on the passed value
         """
         self._xpath = xpath
-        xmlutil.raise_programming_error(not self._xpath,
-                "XMLProperty: xpath must be passed.")
+        if not self._xpath:
+            raise xmlutil.DevError("XMLProperty: xpath must be passed.")
 
         self._is_bool = is_bool
         self._is_int = is_int
@@ -204,8 +233,8 @@ class XMLProperty(_XMLPropertyBase):
         conflicts = sum([int(bool(i)) for i in
                 [self._is_bool, self._is_int,
                  self._is_yesno, self._is_onoff]])
-        xmlutil.raise_programming_error(conflicts > 1,
-                "Conflict property converter options.")
+        if conflicts > 1:
+            raise xmlutil.DevError("Conflict property converter options.")
 
         self._is_tracked = False
         if _trackprops:
@@ -436,7 +465,7 @@ class XMLBuilder(object):
     # Name of the root XML element
     XML_NAME = None
 
-    # In some cases, libvirt can incorrectly generate unparseable XML.
+    # In some cases, libvirt can incorrectly generate unparsable XML.
     # These are libvirt bugs, but this allows us to work around it in
     # for specific XML classes.
     #
@@ -454,14 +483,16 @@ class XMLBuilder(object):
         # know don't work
         forbid = [" "]
         if not val:
-            raise ValueError(
-                _("A name must be specified for the %s") % name_label)
+            # translators: value is a generic object type name
+            raise ValueError(_("A name must be specified for the %s") %
+                    name_label)
         for c in forbid:
             if c not in val:
                 continue
-            raise ValueError(
-                _("%s name '%s' can not contain '%s' character.") %
-                (name_label, val, c))
+            msg = (_("%(objecttype)s name '%(name)s' can not contain "
+                    "'%(char)s' character.") %
+                    {"objecttype": name_label, "name": val, "char": c})
+            raise ValueError(msg)
 
 
     def __init__(self, conn, parsexml=None,
@@ -487,6 +518,7 @@ class XMLBuilder(object):
 
         self._validate_xmlbuilder()
         self._initial_child_parse()
+        self._manual_actions = []
 
     def _validate_xmlbuilder(self):
         # This is one time validation we run once per XMLBuilder class
@@ -497,16 +529,16 @@ class XMLBuilder(object):
         xmlprops = self._all_xml_props()
         childprops = self._all_child_props()
         for key in self._XML_PROP_ORDER:
-            xmlutil.raise_programming_error(
-                key not in xmlprops and key not in childprops,
-                "key '%s' must be xml prop or child prop" % key)
+            if key not in xmlprops and key not in childprops:
+                raise xmlutil.DevError(
+                        "key '%s' must be xml prop or child prop" % key)
 
         childclasses = []
         for childprop in childprops.values():
-            xmlutil.raise_programming_error(
-                childprop.child_class in childclasses,
-                "can't register duplicate child_class=%s" %
-                childprop.child_class)
+            if childprop.child_class in childclasses:
+                raise xmlutil.DevError(
+                        "can't register duplicate child_class=%s" %
+                        childprop.child_class)
             childclasses.append(childprop.child_class)
 
         setattr(self.__class__, cachekey, True)
@@ -613,6 +645,13 @@ class XMLBuilder(object):
             return 0
         return int(xpath.rsplit("[", 1)[1].strip("]")) - 1
 
+    def add_xml_manual_action(self, manualaction):
+        """
+        Register a manual XML action to perform at the end of the
+        XML building step. Triggered via --xml on the command line
+        """
+        self._manual_actions.append(manualaction)
+
 
     ################
     # Internal API #
@@ -639,7 +678,8 @@ class XMLBuilder(object):
             if child_class is xmlprop.child_class:
                 ret = xmlprop
                 break
-        xmlutil.raise_programming_error(not ret,
+        if not ret:
+            raise xmlutil.DevError(
                 "Didn't find child property for child_class=%s" % child_class)
         return ret
 
@@ -793,3 +833,6 @@ class XMLBuilder(object):
             elif key in childprops:
                 for obj in xmlutil.listify(getattr(self, key)):
                     obj._add_parse_bits(self._xmlstate.xmlapi)
+
+        for manualaction in self._manual_actions:
+            manualaction.perform(self._xmlstate)
